@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2005 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #  Copyright (C) 2003 by Shun-ichi TAHARA <jado@flowernet.gr.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
@@ -323,6 +323,9 @@ class Ghost:
     def set_raise_before_talk(self, flag):
         self.__sakura.set_raise_before_talk(flag)
 
+    def set_animation_quality(self, quality):
+        self.__surface.set_animation_quality(quality)
+
     def set_balloon_position(self, side, x, y): ## FIXME
         self.__balloon.set_position(side, x, y)
 
@@ -406,6 +409,8 @@ class Ghost:
         self.communicate.rebuild_ghostdb(self, None)
         self.hide_all()
         self.__sakura.stop()
+        self.__surface.finalize()
+        self.__balloon.finalize()
 
     def restart(self):
         self.load_history()
@@ -450,11 +455,9 @@ class Ghost:
         self.__sakura.set_mouse_button3(button3)
 
     def finalize(self):
-        self.stop()
         if not self.__temp_mode:
             self.unload_shiori()
-        self.__surface.finalize()
-        self.__balloon.finalize()
+        self.stop()
 
     def get_vanished_count(self):
         return self.vanished_count
