@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 #
-#  Copyright (C) 2004, 2005 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -95,6 +95,7 @@ class Kinoko:
 
     def __init__(self, skin_list):
         self.skin_list = skin_list
+        self.skin = None
 
     def edit_preferences(self, action):
         pass
@@ -105,6 +106,8 @@ class Kinoko:
         self.skin.destroy()
 
     def notify_observer(self, event, args): ## FIXME
+        if self.skin is None:
+            return
         if event in ['set position', 'set surface']:
             self.skin.set_position()
             self.skin.show()
@@ -197,6 +200,7 @@ class Skin:
         self.__menu = Menu(self.kinoko, self.accelgroup)
         self.__scale = scale
         self.__shown = 0
+        self.surface_id = 0 # dummy
         self.window = gtk.Window()
         ##self.window.set_title(''.join(('surface.', name)))
         self.window.set_decorated(False)
@@ -217,7 +221,13 @@ class Skin:
             self.seriko = ninix.seriko.Controler(
                 {'': ninix.seriko.get_actors(ninix.config.open(path))})
         else:
-            self.seriko = ninix.seriko.Controler({'': []})
+            base, ext = os.path.splitext(self.data['base'])
+            path = os.path.join(self.data['dir'], ''.join((base, 'a.txt')))
+            if os.path.exists(path):
+                self.seriko = ninix.seriko.Controler(
+                    {'': ninix.seriko.get_actors(ninix.config.open(path))})
+            else:
+                self.seriko = ninix.seriko.Controler({'': []})
         path = os.path.join(self.data['dir'], self.data['base'])
         try:
             self.pixbuf = ninix.pix.create_pixbuf_from_file(path)
@@ -247,6 +257,7 @@ class Skin:
         self.darea.window.set_back_pixmap(image, False)
         self.show()
         self.seriko.reset(self, '') # XXX
+        self.seriko.update(self)
         self.seriko.invoke_kinoko(self)
         if self.data['ontop']:
             self.window.set_keep_above(True)
@@ -263,6 +274,12 @@ class Skin:
             self.window.hide()
             self.__shown = 0
 
+    def prefetch(self, surface_ids):
+        pass
+
+    def append_actor(self, frame, actor):
+        self.seriko.append_actor(frame, actor)
+
     def set_position(self, xoffset=0, yoffset=0): ## FIXME
         base_x, base_y = self.kinoko.target.get_kinoko_position(
             self.data['baseposition'])
@@ -279,7 +296,7 @@ class Skin:
 
     def set_scale(self, scale):
         self.__scale = scale
-        self.draw_overlays() ## FIXME
+        self.udpate()
         self.set_position()
 
     def get_surface(self): ## FIXME
@@ -296,40 +313,25 @@ class Skin:
         self.darea.queue_draw()
 
     def get_pixbuf(self, surface_id):
-        path = os.path.join(self.data['dir'], ''.join(('surface', surface_id, '.png')))
-        pixbuf = ninix.pix.create_pixbuf_from_file(path)
+        path = os.path.join(self.data['dir'], ''.join(('surface', str(surface_id), '.png')))
+        if os.path.exists(path):
+            pixbuf = ninix.pix.create_pixbuf_from_file(path)
+        else:
+            pixbuf = None
         return pixbuf
 
-    def draw_overlays(self): ## FIXME
-        surface_pixbuf = self.pixbuf.copy()
-        for surface_id, x, y in self.seriko.iter_overlays():
-            try:
-                pixbuf = self.get_pixbuf(surface_id)
-                w = pixbuf.get_width()
-                h = pixbuf.get_height()
-            except:
-                print 'cannot load surface #%d' % surface_id
-                continue
-            # overlay surface pixbuf
-            sw = surface_pixbuf.get_width()
-            sh = surface_pixbuf.get_height()
-            if x + w > sw:
-                w = sw - x
-            if y + h > sh:
-                h = sh - y
-            if x < 0:
-                dest_x = 0
-                w += x
-            else:
-                dest_x = x
-            if y < 0:
-                dest_y = 0
-                h += y
-            else:
-                dest_y = y
-            pixbuf.composite(
-                surface_pixbuf, dest_x, dest_y, w, h, x, y, 1.0, 1.0,
-                gtk.gdk.INTERP_BILINEAR, 255)
+    def create_surface_pixbuf(self, surface_id=None):
+        if surface_id is not None and surface_id != '':
+            surface_pixbuf = self.get_pixbuf(surface_id)
+        else:
+            surface_pixbuf = self.pixbuf.copy()
+        return surface_pixbuf
+
+    def update(self, cache=None):
+        if cache is not None:
+            surface_pixbuf = cache
+        else:
+            surface_pixbuf = self.create_surface_pixbuf()
         if self.__scale != 100:
             w = max(1, surface_pixbuf.get_width() * self.__scale / 100)
             h = max(1, surface_pixbuf.get_height() * self.__scale / 100)
@@ -354,7 +356,7 @@ class Skin:
 
     def set_surface(self, surface_id, restart=1): ## FIXME
         self.pixbuf = self.get_pixbuf(surface_id)
-        self.draw_overlays()
+        self.update()
 
     def invoke(self, actor_id, update=0):
         self.seriko.invoke(self, actor_id, update)
