@@ -13,7 +13,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: main.py,v 1.136 2006/08/26 16:01:18 shy Exp $
+# $Id: main.py,v 1.138 2006/10/05 15:16:44 shy Exp $
 #
 
 import getopt
@@ -262,6 +262,7 @@ class Application:
     PREFS_SURFACE_ALPHA     = 'surface_alpha'
     PREFS_BALLOON_ALPHA     = 'balloon_alpha'
     PREFS_RAISE_BEFORE_TALK = 'raise_before_talk'
+    PREFS_ANIMATION_QUALITY = 'animation_quality'
 
     def __init__(self, config, sstp_port=[9801, 11000], debug=0):
         self.ghosts, self.shells, self.balloons, self.plugins, \
@@ -514,7 +515,7 @@ class Application:
             ghost.set_surface_alpha(alpha)
 
     def get_surface_alpha(self):
-        return self.prefs.getint(self.PREFS_SURFACE_ALPHA)
+        return self.prefs.getfloat(self.PREFS_SURFACE_ALPHA)
 
     def set_balloon_alpha(self, alpha):
         self.prefs[self.PREFS_BALLOON_ALPHA] = str(alpha)
@@ -522,7 +523,15 @@ class Application:
             ghost.set_balloon_alpha(alpha)
 
     def get_balloon_alpha(self):
-        return self.prefs.getint(self.PREFS_BALLOON_ALPHA)
+        return self.prefs.getfloat(self.PREFS_BALLOON_ALPHA)
+
+    def set_animation_quality(self, quality):
+        self.prefs[self.PREFS_ANIMATION_QUALITY] = str(quality)
+        for ghost in self.get_working_ghost():
+            ghost.set_animation_quality(quality)
+
+    def get_animation_quality(self):
+        return self.prefs.getfloat(self.PREFS_ANIMATION_QUALITY)
 
     def get_ghost_list(self):
         return self.ghost_list
@@ -949,6 +958,7 @@ class Application:
         ghost.set_script_speed(self.get_script_speed())
         ghost.start(item, init, temp, vanished, ghost_changed,
                     self.get_ignore_default(), name)
+        ghost.set_animation_quality(self.get_animation_quality())
 
     def start_sakura_cb(self, event, item):
         self.start_sakura(item, init=1)
@@ -1092,6 +1102,7 @@ class Application:
             self.get_surface_alpha(),
             self.get_balloon_alpha(),
             self.get_raise_before_talk(),
+            self.get_animation_quality(),
             ]
         self.pref_dialog.set_event_kill_list(self.saved_prefs[0])
         self.pref_dialog.set_balloon_fonts(self.saved_prefs[1])
@@ -1113,6 +1124,7 @@ class Application:
         self.pref_dialog.set_surface_alpha(self.saved_prefs[17])
         self.pref_dialog.set_balloon_alpha(self.saved_prefs[18])
         self.pref_dialog.set_raise_before_talk(self.saved_prefs[19])
+        self.pref_dialog.set_animation_quality(self.saved_prefs[20])
         self.pref_dialog.show()
 
     def notify_preferences_changed(self, done):
@@ -1136,6 +1148,7 @@ class Application:
         self.set_surface_alpha(self.pref_dialog.get_surface_alpha())
         self.set_balloon_alpha(self.pref_dialog.get_balloon_alpha())
         self.set_raise_before_talk(self.pref_dialog.get_raise_before_talk())
+        self.set_animation_quality(self.pref_dialog.get_animation_quality())
 
     def notify_preferences_reverted(self):
         self.set_event_kill_list(self.saved_prefs[0])
@@ -1158,6 +1171,7 @@ class Application:
         self.set_surface_alpha(self.saved_prefs[17])
         self.set_balloon_alpha(self.saved_prefs[18])
         self.set_raise_before_talk(self.saved_prefs[19])
+        self.set_animation_quality(self.saved_prefs[20])
 
     def show_usage(self):
         for ghost in self.get_working_ghost():
@@ -1560,6 +1574,25 @@ class PreferenceDialog:
         button = gtk.SpinButton(self.balloon_alpha_adjustment, 0.2, 1)
         hbox.pack_start(button, False)
         button.show()
+        frame = gtk.Frame(unicode(_('Animation'), 'utf-8'))
+        frame.set_size_request(480, -1)
+        page.pack_start(frame, False)
+        frame.show()
+        box = gtk.VBox(spacing=5)
+        box.set_border_width(5)
+        frame.add(box)
+        box.show()
+        hbox = gtk.HBox(spacing=5)
+        box.add(hbox)
+        hbox.show()
+        label = gtk.Label(unicode(_("Quality"), 'utf-8'))
+        hbox.pack_start(label, False)
+        label.show()        
+        self.animation_quality_adjustment = gtk.Adjustment(1.0, 0.1, 1.0, 0.1, 0.1)
+        button = gtk.SpinButton(self.animation_quality_adjustment, 0.2, 1)
+        hbox.pack_start(button, False)
+        button.show()
+        hbox.show()
         return page        
 
     def make_page_misc(self):
@@ -1933,6 +1966,15 @@ class PreferenceDialog:
         alpha = self.balloon_alpha_adjustment.get_value()
         return float(alpha)
 
+    def set_animation_quality(self, quality):
+        if quality is None:
+            quality = 1.0
+        self.animation_quality_adjustment.set_value(quality)
+
+    def get_animation_quality(self):
+        quality = self.animation_quality_adjustment.get_value()
+        return float(quality)
+
 class RuleEditor:
 
     def __init__(self, master=None):
