@@ -96,9 +96,7 @@ def add_alpha_from_PNA_file(pixbuf, path):
     pixbuf_array = pixbuf.get_pixels_array()
     width = pixbuf.get_width()
     height = pixbuf.get_height()
-    for i in range(height):
-        for j in range(width):
-            pixbuf_array[i][j][3] = pna_array[i][j][0]
+    pixbuf_array[:,:,3] = pna_array[:,:,0]
 
 def create_pixbuf_from_file(path, is_pnr=1, use_pna=0):
     head, tail = os.path.split(path)
