@@ -2,7 +2,7 @@
 #
 #  Copyright (C) 2002 by Tamito KAJIYAMA
 #  Copyright (C) 2002, 2003 by MATSUMURA Namihiko <nie@counterghost.net>
-#  Copyright (C) 2002-2005 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2006 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -11,13 +11,14 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: seriko.py,v 1.23 2005/10/06 02:59:13 shy Exp $
+# $Id: seriko.py,v 1.25 2006/10/05 15:16:44 shy Exp $
 #
 
 import re
 import sys
 import random
 
+import gtk
 import gobject
 
 def print_error(message):
@@ -30,28 +31,153 @@ class Controler:
         self.seriko = seriko
         self.exclusive_actor = None
         self.base_id = None
+        self.timeout_id = None
+        self.idle_id = None
         self.reset_overlays()
         self.queue = []
+        self.frame = {'step': 0.033, # [sec]
+                      'next': 0}
+        self.active = []
+        self.__cache = []
+        self.__move = None
+        self.__dirty = 1
+
+    def cache_is_empty(self):
+        if self.__cache:
+            return False
+        else:
+            return True
+
+    def get_cache(self):
+        assert self.__cache is not None
+        return self.__cache[0][2]
+
+    def set_base_id(self, window, surface_id):
+        if surface_id == -2:
+            self.terminate()
+            self.base_id = window.surface_id
+        else:
+            self.base_id = surface_id
+        self.__dirty = 1
+
+    def move_surface(self, xoffset, yoffset):
+        self.__move = (xoffset, yoffset)
+
+    def draw_overlays(self, window, surface_pixbuf):
+        ##print 'draw_overlays()'
+        if surface_pixbuf is None:
+            return None
+        for pixbuf_id, x, y in self.iter_overlays():
+            try:
+                pixbuf = window.get_pixbuf(pixbuf_id)
+                w = pixbuf.get_width()
+                h = pixbuf.get_height()
+            except:
+                continue
+            # overlay surface pixbuf
+            sw = surface_pixbuf.get_width()
+            sh = surface_pixbuf.get_height()
+            if x + w > sw:
+                w = sw - x
+            if y + h > sh:
+                h = sh - y
+            if x < 0:
+                dest_x = 0
+                w += x
+            else:
+                dest_x = x
+            if y < 0:
+                dest_y = 0
+                h += y
+            else:
+                dest_y = y
+            pixbuf.composite(surface_pixbuf, dest_x, dest_y,
+                             w, h, x, y, 1.0, 1.0,
+                             gtk.gdk.INTERP_BILINEAR, 255)
+        return surface_pixbuf
+
+    def append_actor(self, frame, actor):
+        self.active.append((frame, actor))
+
+    def set_animation_quality(self, quality, window):
+        if self.timeout_id is not None:
+            gobject.source_remove(self.timeout_id)
+        if quality is None:
+            quality = 1.0
+        step = 0.033 / quality ## FIXME
+        self.frame['step'] = step
+        if self.timeout_id is not None:
+            self.timeout_id = gobject.timeout_add(int(step * 1000.0), self.update, window)
+
+    def update_frame(self, window):
+        if self.terminate_flag:
+            return False
+        if len(self.__cache) > 1.0 / self.frame['step']: # XXX
+            return True
+        self.frame['next'] += self.frame['step']
+        frame, actor = self.get_actor_next(window)
+        last_actor = actor
+        while actor is not None:
+            actor.update(window, frame)
+            last_actor = actor
+            frame, actor = self.get_actor_next(window)
+        self.update_surface_cache(window)
+        if last_actor is not None and last_actor.exclusive and \
+           last_actor.terminate_flag and self.exclusive_actor is None: # XXX
+            self.invoke_restart(window)
+        self.idle_id = gobject.timeout_add(10, self.update_frame, window) # XXX
+        return False
+
+    def get_actor_next(self, window):
+        if self.active:
+            self.active.sort()
+            if self.active[0][0] <= self.frame['next']:
+                return self.active.pop(0)
+        return None, None
+
+    def update_surface_cache(self, window):
+        if self.__dirty:
+            surface_pixbuf = window.create_surface_pixbuf(self.base_id)
+            surface_pixbuf = self.draw_overlays(window, surface_pixbuf)
+        else:
+            surface_pixbuf = None
+        self.__cache.append((self.frame['next'], self.__move, surface_pixbuf, self.__dirty))
+        self.__dirty = 0
+
+    def update(self, window):
+        if self.terminate_flag:
+            return False
+        if self.__cache:
+            frame, move, surface, dirty = self.__cache.pop(0)
+            if move is not None:
+                window.move_surface(*move)
+            if dirty:
+                window.update(surface)
+        self.timeout_id = gobject.timeout_add(int(self.frame['step'] * 1000), self.update, window)
+        return False
 
     def lock_exclusive(self, window, actor):
         assert self.exclusive_actor is None
         self.terminate()
         self.exclusive_actor = actor
         actor.set_post_proc(self.unlock_exclusive, (window, actor))
+        self.update_frame(window) # XXX
+        self.__dirty = 1
 
     def unlock_exclusive(self, window, actor):
         assert self.exclusive_actor == actor
         self.exclusive_actor = None
-        self.invoke_restart(window)
 
     def reset_overlays(self):
         self.overlays = {}
+        self.__dirty = 1
 
     def remove_overlay(self, actor):
         try:
             del self.overlays[actor]
         except KeyError:
             pass
+        self.__dirty = 1
 
     def add_overlay(self, actor, pixbuf_id, x, y):
         if pixbuf_id == '-2':
@@ -60,6 +186,7 @@ class Controler:
             self.remove_overlay(actor)
             return
         self.overlays[actor] = (pixbuf_id, x, y)
+        self.__dirty = 1
 
     def invoke_actor(self, window, actor):
         if self.exclusive_actor is not None:
@@ -70,7 +197,7 @@ class Controler:
             return
         if actor.exclusive:
             self.lock_exclusive(window, actor)
-        actor.invoke(window)
+        actor.invoke(window, self.frame['next'])
 
     def invoke(self, window, actor_id, update=0):
         if self.base_id not in self.seriko:
@@ -138,6 +265,8 @@ class Controler:
     def reset(self, window, surface_id):
         self.queue = []
         self.terminate()
+        self.frame['next'] = 0
+        self.set_base_id(window, window.surface_id)
         if surface_id in self.seriko:
             self.base_id = surface_id
         else:
@@ -150,16 +279,30 @@ class Controler:
                         surface_ids.append(surface_id)
             if surface_ids:
                 window.prefetch(surface_ids)
+        self.update_frame(window)
 
     def start(self, window):
         self.invoke_runonce(window)
         self.invoke_always(window)
+        self.timeout_id = gobject.timeout_add(int(self.frame['step'] * 1000), self.update, window)
 
     def terminate(self):
+        self.terminate_flag = 1
+        if self.timeout_id is not None:
+            gobject.source_remove(self.timeout_id)
+            self.timeout_id = None
+        if self.idle_id is not None:
+            gobject.source_remove(self.idle_id)
+            self.idle_id = None
         if self.base_id in self.seriko:
             for actor in self.seriko[self.base_id]:
                 actor.terminate()
         self.reset_overlays()
+        self.active = []
+        self.__move = None
+        self.__dirty = 1
+        self.__cache = []
+        self.terminate_flag = 0
 
     def iter_overlays(self):
         actors = self.overlays.keys()
@@ -179,7 +322,6 @@ class Actor:
         self.patterns = []
         self.last_method = None
         self.exclusive = 0
-        self.timeout_id = None
         self.post_proc = None
         self.terminate_flag = 1
 
@@ -202,19 +344,16 @@ class Actor:
     def add_pattern(self, surface, interval, method, args):
         self.patterns.append((surface, interval, method, args))
 
-    def invoke(self):
+    def invoke(self, base_frame):
         self.terminate_flag = 0
 
-    def update(self, window):
+    def update(self, window, base_frame):
         if self.terminate_flag:
             return False
         pass
 
     def terminate(self):
         self.terminate_flag = 1
-        if self.timeout_id is not None:
-            gobject.source_remove(self.timeout_id)
-            self.timeout_id = None
         if self.post_proc is not None:
             proc, args = self.post_proc
             self.post_proc = None
@@ -231,12 +370,11 @@ class Actor:
         if self.last_method in ['overlay', 'overlayfast']:
             window.remove_overlay(self)
         if method == 'move':
-            window.move_surface(args[0], args[1])
+            window.seriko.move_surface(args[0], args[1]) ## FIXME
         elif method in ['overlay', 'overlayfast']:
             window.add_overlay(self, surface, args[0], args[1])
         elif method == 'base':
-            window.surface_id = self.surface_id # This is kluge.
-            window.set_surface(surface, restart=0)
+            window.seriko.set_base_id(window, surface) ## FIXME
         elif method == 'start':
             window.invoke(args[0], update=1)
         elif method == 'alternativestart':
@@ -252,14 +390,13 @@ class ActiveActor(Actor): # always
         self.wait = 0
         self.pattern = 0
 
-    def invoke(self, window):
-        if self.timeout_id is not None:
-            self.terminate()
+    def invoke(self, window, base_frame):
+        self.terminate()
         self.terminate_flag = 0
         self.pattern = 0
-        self.update(window)
+        self.update(window, base_frame)
 
-    def update(self, window):
+    def update(self, window, base_frame):
         if self.terminate_flag:
             return False
         if self.pattern == 0:
@@ -269,7 +406,7 @@ class ActiveActor(Actor): # always
         if self.pattern == len(self.patterns):
             self.pattern = 0
         self.show_pattern(window, surface, method, args)
-        self.timeout_id = gobject.timeout_add(interval, self.update, window)
+        window.append_actor(base_frame + interval / 1000.0, self)
         return False
 
 class RandomActor(Actor): # sometimes, rarely, randome
@@ -284,14 +421,13 @@ class RandomActor(Actor): # sometimes, rarely, randome
         self.wait = random.randint(self.wait_min, self.wait_max)
         self.pattern = 0
 
-    def invoke(self, window):
-        if self.timeout_id is not None:
-            self.terminate()
+    def invoke(self, window, base_frame):
+        self.terminate()
         self.terminate_flag = 0
         self.reset()
-        self.timeout_id = gobject.timeout_add(self.wait, self.update, window)
+        window.append_actor(base_frame + self.wait / 1000.0, self)
 
-    def update(self, window):
+    def update(self, window, base_frame):
         if self.terminate_flag:
             return False
         if self.pattern == 0:
@@ -303,7 +439,7 @@ class RandomActor(Actor): # sometimes, rarely, randome
         else:
             self.reset()
         self.show_pattern(window, surface, method, args)
-        self.timeout_id = gobject.timeout_add(self.wait, self.update, window)
+        window.append_actor(base_frame + self.wait / 1000.0, self)
         return False
 
 class OneTimeActor(Actor): # runone
@@ -313,15 +449,14 @@ class OneTimeActor(Actor): # runone
         self.wait = -1
         self.pattern = 0
 
-    def invoke(self, window):
-        if self.timeout_id is not None:
-            self.terminate()
+    def invoke(self, window, base_frame):
+        self.terminate()
         self.terminate_flag = 0
         self.wait = 0
         self.pattern = 0
-        self.update(window)
+        self.update(window, base_frame)
 
-    def update(self, window):
+    def update(self, window, base_frame):
         if self.terminate_flag:
             return False
         if self.pattern == 0:
@@ -335,7 +470,7 @@ class OneTimeActor(Actor): # runone
             self.terminate()
         self.show_pattern(window, surface, method, args)
         if self.wait >= 0:
-            self.timeout_id = gobject.timeout_add(self.wait, self.update, window)
+            window.append_actor(base_frame + self.wait / 1000.0, self)
         return False
 
 class PassiveActor(Actor): # never, yen-e, talk
@@ -344,15 +479,14 @@ class PassiveActor(Actor): # never, yen-e, talk
         Actor.__init__(self, actor_id, interval)
         self.wait = -1
 
-    def invoke(self, window):
-        if self.timeout_id is not None:
-            self.terminate()
+    def invoke(self, window, base_frame):
+        self.terminate()
         self.terminate_flag = 0
         self.wait = 0
         self.pattern = 0
-        self.update(window)
+        self.update(window, base_frame)
 
-    def update(self, window):
+    def update(self, window, base_frame):
         if self.terminate_flag:
             return False
         if self.pattern == 0:
@@ -366,7 +500,7 @@ class PassiveActor(Actor): # never, yen-e, talk
             self.terminate()
         self.show_pattern(window, surface, method, args)
         if self.wait >= 0:
-            self.timeout_id = gobject.timeout_add(self.wait, self.update, window)
+            window.append_actor(base_frame + self.wait / 1000.0, self)
         return False
 
 class Mayuna(Actor):
