@@ -93,6 +93,10 @@ class Surface:
             surface_window.set_scale(scale)
             surface_window.reset_surface()
 
+    def set_animation_quality(self, quality):
+        for surface_window in self.window:
+            surface_window.set_animation_quality(quality)
+
     def finalize(self):
         for surface_window in self.window:
             surface_window.destroy()
@@ -876,6 +880,9 @@ class SurfaceWindow:
         ##    return
         self.__scale = scale # %
 
+    def set_animation_quality(self, quality):
+        self.seriko.set_animation_quality(quality, self)
+
     def drag_data_received(self, widget, context, x, y, data, info, time):
         if self.debug & 8192:
             print 'Content-type:', data.type
@@ -892,6 +899,9 @@ class SurfaceWindow:
                 self.sakura.notify_file_drop(filelist, self.side)
         return True
 
+    def append_actor(self, frame, actor):
+        self.seriko.append_actor(frame, actor)
+
     def invoke(self, actor_id, update=0):
         self.seriko.invoke(self, actor_id, update)
 
@@ -903,17 +913,17 @@ class SurfaceWindow:
 
     def reset_surface(self):
         surface_id = self.get_surface()
-        self.set_surface(surface_id, restart=0)
+        self.set_surface(surface_id)
 
     def set_surface_default(self):
         self.set_surface(self.default_id)
 
-    def set_surface(self, surface_id, restart=1):
+    def set_surface(self, surface_id):
         if self.alias is not None and surface_id in self.alias:
             aliases = self.alias.get(surface_id)
             if aliases:
                 surface_id = random.choice(aliases)
-        if surface_id == '-2' or restart:
+        if surface_id == '-2':
             self.seriko.terminate()
         if surface_id in ['-1', '-2']:
             pass
@@ -921,8 +931,7 @@ class SurfaceWindow:
             self.surface_id = self.default_id
         else:
             self.surface_id = surface_id
-        if restart:
-            self.seriko.reset(self, surface_id)
+        self.seriko.reset(self, surface_id)
         # define collision areas
         self.collisions = self.region[self.surface_id]
         # update window offset
@@ -944,8 +953,7 @@ class SurfaceWindow:
         # resize window
         self.darea.set_size_request(w, h)
         self.update()
-        if restart:
-            self.seriko.start(self)
+        self.seriko.start(self)
         # relocate window
         self.set_position(x, y)
         self.sakura.ghost.notify_surface_change(self.side) ## FIXME
@@ -1065,11 +1073,12 @@ class SurfaceWindow:
         if pixbuf_id not in self.pixbuf:
             return pix.create_blank_pixbuf(8, 8)
         if pixbuf_id in self.pixbuf_cache:
-            pixbuf = self.pixbuf_cache[pixbuf_id]
+            pixbuf = self.pixbuf_cache[pixbuf_id][0]
         else:
             pixbuf = self.create_pixbuf_from_file(pixbuf_id)
-            self.pixbuf_cache[pixbuf_id] = pixbuf
-        return pixbuf
+            self.pixbuf_cache[pixbuf_id] = (pixbuf, (
+                pixbuf.get_width(), pixbuf.get_height()))
+        return pixbuf.copy()
 
     def draw_region(self):
         surface_gc = self.darea.window.new_gc()
@@ -1083,24 +1092,32 @@ class SurfaceWindow:
             self.darea.window.draw_rectangle(
                 surface_gc, 0, x1, y1, x2 - x1, y2 - y1)
 
-    def update(self):
-        if self.surface_id in self.mayuna:
-            if self.surface_id in self.mayuna_cache:
-                surface_pixbuf = self.mayuna_cache[self.surface_id].copy()
+    def create_surface_pixbuf(self, surface_id=None):
+        if surface_id is None:
+            surface_id = self.surface_id
+        if surface_id in self.mayuna and self.mayuna[surface_id]:
+            if surface_id in self.mayuna_cache:
+                surface_pixbuf = self.mayuna_cache[surface_id].copy()
             else:
-                surface_pixbuf = self.surface_pixbuf.copy()
+                surface_pixbuf = self.get_pixbuf(surface_id)
                 done = []
-                for actor in self.mayuna[self.surface_id]:
+                for actor in self.mayuna[surface_id]:
                     actor_id = actor.get_id()
                     if actor_id in self.bind and self.bind[actor_id][1] and \
                        actor_id not in done:
                         done.append(actor_id)
                         surface_pixbuf = self.compose_surface(
                             surface_pixbuf, actor, done)
-                self.mayuna_cache[self.surface_id] = surface_pixbuf.copy()
+                self.mayuna_cache[surface_id] = surface_pixbuf.copy()
         else:
-            surface_pixbuf = self.surface_pixbuf.copy()
-        surface_pixbuf = self.draw_overlays(surface_pixbuf)
+            surface_pixbuf =self.get_pixbuf(surface_id)
+        return surface_pixbuf
+
+    def update(self, cache=None):
+        if cache is not None:
+            surface_pixbuf = cache
+        else:
+            surface_pixbuf = self.create_surface_pixbuf()
         if self.__scale != 100:
             w = max(1, surface_pixbuf.get_width() * self.__scale / 100)
             h = max(1, surface_pixbuf.get_height() * self.__scale / 100)
@@ -1127,40 +1144,6 @@ class SurfaceWindow:
 
     def add_overlay(self, actor, pixbuf_id, x, y):
         self.seriko.add_overlay(actor, pixbuf_id, x, y)
-        self.update()
-
-    def draw_overlays(self, surface_pixbuf):
-        ##print 'draw_overlays()'
-        if surface_pixbuf is None:
-            return None
-        for pixbuf_id, x, y in self.seriko.iter_overlays():
-            try:
-                pixbuf = self.get_pixbuf(pixbuf_id)
-                w = pixbuf.get_width()
-                h = pixbuf.get_height()
-            except:
-                continue
-            # overlay surface pixbuf
-            sw = surface_pixbuf.get_width()
-            sh = surface_pixbuf.get_height()
-            if x + w > sw:
-                w = sw - x
-            if y + h > sh:
-                h = sh - y
-            if x < 0:
-                dest_x = 0
-                w += x
-            else:
-                dest_x = x
-            if y < 0:
-                dest_y = 0
-                h += y
-            else:
-                dest_y = y
-            pixbuf.composite(surface_pixbuf, dest_x, dest_y,
-                             w, h, x, y, 1.0, 1.0,
-                             gtk.gdk.INTERP_BILINEAR, 255)
-        return surface_pixbuf
 
     def __move(self, xoffset=0, yoffset=0):
         x, y = self.get_position()
@@ -1186,11 +1169,12 @@ class SurfaceWindow:
 
     def get_surface_size(self, surface_id=None):
         if surface_id is None:
-            pixbuf = self.get_pixbuf(self.surface_id)
-        else:
-            pixbuf = self.get_pixbuf(surface_id)
-        w = pixbuf.get_width()
-        h = pixbuf.get_height()
+            surface_id = self.surface_id
+        if surface_id in self.pixbuf_cache:
+            w, h = self.pixbuf_cache[surface_id][1]
+        else: # XXX
+            self.get_pixbuf(surface_id)
+            w, h = self.pixbuf_cache[surface_id][1]
         if self.__scale != 100:
             w = int(w * self.__scale / 100)
             h = int(h * self.__scale / 100)
@@ -1253,6 +1237,7 @@ class SurfaceWindow:
         self.reset_pixbuf_cache()
         for tag in self.callbacks:
             self.darea.disconnect(tag)
+        self.seriko.terminate()
         self.window.remove(self.darea)
         self.darea.destroy()
         self.window.destroy()
@@ -1352,7 +1337,7 @@ class SurfaceWindow:
         if bind_id in self.bind:
             current = self.bind[bind_id][1]
             self.bind[bind_id][1] = not current
-            self.mayuna_cache = {}
+            self.reset_pixbuf_cache()
             self.reset_surface()
 
     def window_leave_notify(self, window, event):
