@@ -10,7 +10,7 @@
 #  PURPOSE.  See the GNU General Public License for more details.
 #
 
-NUMBER = '3.9'
+NUMBER = '3.9.3'
 CODENAME = '"I see no more YASAGURE here."'
 
 VERSION = '%s (%s)' % (NUMBER, CODENAME)
